<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 *  Controllerクラス
 *
 *  @author     Shogo Minami
 *  @access     public
 *  @package    Pocket
 */
class Pocket_Controller
{
    /** @var    string  モジュール名 */
    var $_module;

    /** @var    string  アクション名 */
    var $_action;

    /** @var    string  タイプ名 */
    var $_type;

    /**
     *  コンストラクタ
     *
     *  @access public
     */
    function Pocket_Controller()
    {
        // モジュール名を取得
        $this->_module = pocket_getParameter('m');
        // アクション名、タイプ名を取得
        $at = explode('.', pocket_getParameter('a'), 2);
        if (!empty($at[0]))
        {
            $this->_action = $at[0];
        }
        if (!empty($at[1]))
        {
            $this->_type = $at[1];
        }

        // モジュール名の存在チェック
        if (!$this->_module)
        {
            // User-Agentを判定
            $UAJudgment = new UAJudgment();

            if ($UAJudgment->isMobile())
            {
                $this->_module = 'mobile';
            }
            else {
                $this->_module = 'pc';
            }
        }

        // アクション名をチェック
        if (!$this->_action)
        {
            $this->_action = 'default';
        }

        // タイプ名をチェック
        if (!$this->_type)
        {
            $this->_type = 'view';
        }
    }

    /**
     *  ディスパッチ
     *
     *  @access public
     *  @param  string  $modName            モジュール名
     *  @param  string  $actName            アクション名
     *  @param  string  $typName            タイプ名
     */
    function dispatch($modName = NULL, $actName = NULL, $typName = NULL)
    {
        if ($modName != NULL)
        {
            $this->_module = $modName;
        }
        if ($actName != NULL)
        {
            $this->_action = $actName;
        }
        if ($typName != NULL)
        {
            $this->_type = $typName;
        }

        $this->forward($this->_module, $this->_action, $this->_type);
    }

    /**
     *  フォワード
     *
     *  @access public
     *  @param  string  $modName            モジュール名
     *  @param  string  $actName            アクション名
     *  @param  string  $typName            タイプ名
     *  @param  array   $errors             エラー情報配列
     */
    function forward($modName, $actName, $typName, $errors = NULL)
    {
        // モジュール名、アクション名、タイプ名を保持
        $GLOBALS['_POCKET_MODULE'] = $modName;
        $GLOBALS['_POCKET_ACTION'] = $actName;
        $GLOBALS['_POCKET_TYPE'] = $typName;

        // モジュールのチェック
        if ($modName != 'setup')
        {
            // 構成定義情報の取得・設定関数の存在チェック
            if (function_exists('pocket_defineConfig'))
            {
                // 構成定義情報の取得・設定
                pocket_defineConfig();
            }
        }

        // 利用可能モジュールか判定
        if (!in_array($modName, $GLOBALS['_POCKET_MODULES']))
        {
            // User-Agentを判定
            $UAJudgment = new UAJudgment();

            if ($UAJudgment->isMobile())
            {
                $GLOBALS['_POCKET_MODULE'] = 'mobile';
            }
            else {
                $GLOBALS['_POCKET_MODULE'] = 'pc';
            }
            pocket_dispError('モジュールの指定が正しくありません。', true);
        }

        // モジュールフィルター名称を生成
        $moduleFilterName = POCKET_MODULES_DIR . $modName . '/moduleFilter.php';

        // モジュールフィルターの読込み
        if (is_readable($moduleFilterName))
        {
            require_once $moduleFilterName;
        }

        // Actionファイル名称を生成
        $actFileName = POCKET_MODULES_DIR . $modName . '/actions/' . $typName . '/' . $actName . 'Action.php';

        // Actionファイルの読込み
        if (is_readable($actFileName))
        {
            require_once $actFileName;
        }
        else {
            pocket_dispError('アクションの指定が正しくありません。', true);
        }

        // モジュールのチェック
        if ($modName != 'setup')
        {
            // アクセスログ出力関数の存在チェック
            if (function_exists('pocket_writeAccessLog'))
            {
                // アクセスログを出力
                pocket_writeAccessLog($modName, $actName, $typName);
            }

            // IPブロック関数の存在チェック
            if (function_exists('pocket_isBlockIP'))
            {
                // IPブロックの判定
                if (pocket_isBlockIP($_SERVER['REMOTE_ADDR']))
                {
                    pocket_dispError('禁止IPに設定されています。');
                }
            }
        }

        // モジュールのチェック
        if ($modName == 'mobile')
        {
            // セッションIDの設定
            if ($request[session_name()])
            {
                session_id($request[session_name()]);
            }
        }

        // セッションの開始
        session_start();

        // セッション情報の保持
        $_SESSION['name'] = session_name();
        $_SESSION['id'] = session_id();

        // リクエスト情報の取得
        $request = pocket_getParameters();

        // Smartyクラスの生成
        $smarty = new Pocket_Smarty;

        // テンプレートディレクトリの設定
        $smarty->template_dir = POCKET_MODULES_DIR . $modName. '/templates';

        // 出力文字コードの設定
        if ($modName == 'mobile')
        {
            $smarty->output_charset = 'SJIS';
        }

        // サイトのURL情報の設定
        $smarty->assign('SITE_URL', pocket_getSiteUrl());
        // URL付加情報の設定
        $smarty->assign('URL_ADDITION', session_name() . '=' . session_id());
        // リクエスト情報の設定
        $smarty->assign('request', $request);

        // エラー情報の存在チェック
        if ($errors)
        {
            // エラー情報の設定
            $smarty->assign('errors', $errors);
        }

        // Actionクラスの生成
        $actClass = $actName . 'Action';
        $actObject = new $actClass();

        // 認証判定関数の存在チェック
        if (function_exists('pocket_isAuth'))
        {
            // 認証アクションフラグのチェック
            if ($actObject->isSecure())
            {
                // 認証チェック
                if (!pocket_isAuth())
                {
                    // リダイレクト
                    pocket_redirect('default.view');
                }
            }

            // 認証情報の設定
            $smarty->assign('is_auth', pocket_isAuth());
        }


        // ActionクラスにSmartyオブジェクトを設定
        $actObject->smarty = & $smarty;

        // Validatorクラスの生成
        $validator = new Pocket_Validator();

        // リクエスト検証
        $actObject->validate($validator, $request);
        if (count($validator->errors))
        {
            // 入力画面へ再フォワード
            $this->forward($modName, $actObject->viewAction, 'view', $validator->errors);
        }
        else {
            // Action実行
            $result = $actObject->execute($this, $request);
            // HTTPヘッダの送信
            header('Content-Type: text/html; charset=' . $smarty->output_charset);
            // テンプレートの表示
            if ($result == POCKET_ACTION_SUCCESS)
            {
                $smarty->display($actName . '.tpl');
            }
            else {
                $smarty->display('error.tpl');
            }
        }
    }
}

?>
