<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 *  Imageクラス
 *
 *  @author     Shogo Minami
 *  @access     public
 *  @package    Pocket
 */
class Pocket_Image
{
    /** @var    string  _cache_dir          キャッシュファイル格納場所 */
    var $_cache_dir;

    /** @var    string  _image_name         画像名称（画像ID） */
    var $_image_name;

    /** @var    string  _image_type         画像ファイル種別 */
    var $_image_type;

    /** @var    binary  _image              画像イメージ */
    var $_image;

    /** @var    string  _cache_file         キャッシュファイル名称 */
    var $_cache_file;

    /** @var    integer _w                  横幅 */
    var $_w;

    /** @var    integer _h                  高さ */
    var $_h;

    /** @var    integer _f                  フォーマット */
    var $_f;

    /**
     *  コンストラクタ
     *
     *  @access public
     *  @param  string  $cache_dir          キャッシュファイル格納場所
     *  @param  string  $image_name         画像名称（画像ID）
     *  @param  string  $image_type         画像ファイル種別
     *  @param  binary  $image              画像イメージ
     */
    function Pocket_Image($cache_dir, $image_name, $image_type, $image)
    {
        // キャッシュファイル格納場所の設定
        $this->_cache_dir = $cache_dir;
        // 画像名称の設定
        $this->_image_name = $image_name;
        // 画像ファイル種別の設定
        $this->_image_type = $this->_getImageType($image_type);
        // 画像イメージの設定
        $this->_image = $image;
        // リクエスト情報の取得
        $this->_getRequest();
    }

    /**
     *  リクエスト情報の取得
     *
     *  @access public
     */
    function _getRequest()
    {
        // 横幅の取得
        $this->_w = $_GET['w'];
        if (!is_numeric($this->_w))
            $this->_w = 0;
        // 高さの取得
        $this->_h = $_GET['h'];
        if (!is_numeric($this->_h))
            $this->_h = 0;
        // フォーマットの取得
        $this->_f = $this->_getImageType($_GET['f']);
        if (!$this->_f)
            $this->_f = $this->_image_type;
    }

    /**
     *  画像ファイル種別の取得
     *
     *  @access public
     *  @param  string  $type               画像ファイル種別
     */
    function _getImageType($type)
    {
        if (strpos($type, 'jpeg') > 0 || strpos($type, 'jpg') > 0)
        {
            return 'jpg';
        }
        elseif (strpos($type, 'gif') > 0)
        {
            return 'gif';
        }
        elseif (strpos($type, 'png') > 0)
        {
            return 'png';
        }
        else {
            return '';
        }
    }

    /**
     *  キャッシュファイル名称の生成
     *
     *  @access public
     */
    function _makeCacheFileName()
    {
        // キャッシュファイル格納場所
        $this->_cache_file = $this->_cache_dir;
        // 画像名称（画像ID）
        if ($this->_image_name)
            $this->_cache_file .= $this->_image_name;
        // 横幅
        if ($this->_w)
            $this->_cache_file .= '_' . $this->_w;
        // 高さ
        if ($this->_h)
            $this->_cache_file .= '_' . $this->_h;
        // 拡張子
        if ($this->_f)
            $this->_cache_file .= '.' . $this->_f;
    }

    /**
     *  画像ファイルの生成（GDのイメージ情報から）
     *
     *  @access public
     *  @param  binary  $image              画像イメージ
     */
    function _createCache4gd($image)
    {
        // ファイルの生成
        touch($this->_cache_file);

        // ファイルへ画像を出力
        switch ($this->_f)
        {
        case 'gif':
            imageGif($image, $this->_cache_file);
            break;
        case 'png':
            imagePNG($image, $this->_cache_file);
            break;
        case 'jpg':
        default:
            imageJPEG($image, $this->_cache_file, 100);
            break;
        }

        // メモリの解放
        ImageDestroy($image);
    }

    /**
     *  画像ファイルの生成（DBのイメージ情報から）
     *
     *  @access public
     *  @param  binary  $image              画像イメージ
     */
    function _createCache4db($image)
    {
        // ファイルへ画像を出力
        switch ($this->_f)
        {
        case 'png':
            if (!$image = imageCreateFromString($image))
            {
                return false;
            }
            touch($this->_cache_file);
            imagePNG($image, $this->_cache_file);
            ImageDestroy($image);
            break;
        default:
            $fp = fopen($this->_cache_file, 'wb');
            fwrite($fp, $image);
            fclose($fp);
            break;
        }

        return true;
    }

    /**
     *  画像ファイルのリサイズ
     *
     *  @access public
     */
    function _resize()
    {
        // GDイメージの生成
        if (!$image = imageCreateFromString($this->_image))
        {
            return false;
        }

        // インターレースを無効にする（携帯対応）
        if ($this->_f == 'jpg')
        {
            imageinterlace($image, 0);
        }

        // イメージのサイズの取得
        $sx = ImageSX($image);
        $sy = ImageSY($image);

        // 指定イメージサイズの補正
        if (!$this->_w) $this->_w = $sx;
        if (!$this->_h) $this->_h = $sy;

        // サイズ変更が必要かチェック
        if ($sx <= $this->_w && $sy <= $this->_h)
        {
            // ファイル種別を変更するかチェック
            if ($this->_f == $this->image_type)
            {
                // 画像ファイルの生成（DBのイメージ情報から）
                return $this->_createCache4db($this->_image);
            }
            else {
                // 画像ファイルの生成（GDのイメージ情報から）
                $this->_createCache4gd($image);
                return true;
            }
        }

        // 出力イメージのサイズの設定
        $ox = $sx;
        $oy = $sy;
        if ($this->_w < $sx)
        {
            $ox = $this->_w;
            $oy = $sy * $this->_w / $sx;
        }
        if ($this->_h < $oy && $this->_h < $sy)
        {
            $ox = $sx * $this->_h / $sy;
            $oy = $this->_h;
        }

        // GDイメージの生成
        $out_image = ImageCreateTrueColor($ox, $oy);
        imageCopyResampled($out_image, $image, 0, 0, 0, 0, $ox, $oy, $sx, $sy);
        // 画像ファイルの生成（GDのイメージ情報から）
        $this->_createCache4gd($out_image);

        // メモリの解放
        ImageDestroy($image);

        return true;
    }

    /**
     *  画像ファイルの生成
     *
     *  @access public
     */
    function createImage()
    {
        // キャッシュファイル格納場所のチェック
        if (!is_dir($this->_cache_dir))
            return false;

        // 画像名称（画像ID）のチェック
        if (!$this->_image_name)
            return false;

        // キャッシュファイル名称の生成
        $this->_makeCacheFileName();

        // キャッシュファイルの存在チェック
        if (is_readable($this->_cache_file))
            return true;

        // 画像ファイル変換の必要性チェック
        if (!$this->_w && !$this->_h && (!$this->_f || $this->_f == $this->_image_type))
        {
            // 画像ファイルの生成（DBのイメージ情報から）
            return $this->_createCache4db($this->_image);
        }

        // 画像ファイルのリサイズ
        return $this->_resize();
    }

    /**
     *  画像ファイルの出力
     *
     *  @access public
     */
    function putImage()
    {
        // ヘッダーの出力
        switch ($this->_f)
        {
        case 'gif':
            header('Content-type: image/gif');
            break;
        case 'png':
            header('Content-type: image/png');
            break;
        case 'jpg':
        default:
            header('Content-type: image/jpeg');
            break;
        }

        // ファイルの読込み＆出力
        @readfile($this->_cache_file);
    }
}

?>
