<?php
/**
 * Pocket Project
 * 
 * @copyright 2006 Power-Station co., Ltd, ALL Rights Reserved,
 */

/**
 *  トピック情報更新用のActionクラス
 *
 *  @author     Shogo Minami
 *  @access     public
 *  @package    PocketBBS.pc.do
 */
class topicUpdateAction extends Pocket_Bbs_Action
{
    /**
     *  アクション実行
     *
     *  @access public
     *  @param  object  $controller         コントローラ
     *  @param  object  $request            リクエスト情報
     *  @return string  success,error
     */
    function execute(&$controller, &$request)
    {
        // リクエスト情報の取得
        $topic_id      = $request['topic_id'];
        $author_name   = $request['author_name'];
        $author_email  = $request['author_email'];
        $subject       = $request['subject'];
        $body          = $request['body'];
        $category_id   = $request['category_id'];
        $allow_comment = $request['allow_comment'];

        // トピックの更新
        db_update_topic($topic_id, $author_name, $author_email, $subject, $body, $category_id, $allow_comment);

        // リダイレクト
        pocket_redirect('topicComment.view', array('topic_id' => $topic_id));
    }

    /**
     *  リクエスト検証
     *
     *  @access public
     *  @param  object  $validator          バリデータ
     *  @param  object  $request            リクエスト情報
     */
    function validate(&$validator, &$request)
    {
        // リクエスト情報の取得
        $topic_id      = $request['topic_id'];
        $author_name   = $request['author_name'];
        $author_email  = $request['author_email'];
        $subject       = $request['subject'];
        $body          = $request['body'];
        $category_id   = $request['category_id'];
        $allow_comment = $request['allow_comment'];

        // エラー表示用のAction名称の設定
        $this->viewAction = 'topicEdit';

        // 検証
        $validator->validate('topic_id'     , 'ID'              , $topic_id     , 'number', true);
        $validator->validate('author_name'  , '名前'            , $author_name  , 'string', true);
        $validator->validate('author_email' , 'E-Mail'          , $author_email , 'email');
        $validator->validate('subject'      , 'タイトル'        , $subject      , 'string', true);
        $validator->validate('category_id'  , 'カテゴリ'        , $category_id  , 'number', true);
        $validator->validate('allow_comment', 'コメント削除許可', $allow_comment, 'number', true);
        $validator->validate('body'         , '本文'            , $body         , 'string', true);

        // トピック情報の存在チェック
        if (!db_get_topic_info($topic_id))
            pocket_redirect('default.view');

        // 認証チェック
        if (!pocket_isAuth())
        {
            // リクエスト情報の取得
            $topic_id        = $request['topic_id'];
            $author_password = $request['author_password'];

            // エラー表示用のAction名称の設定
            $this->viewAction = 'topicEditConfirm';

            // 検証
            $validator->validate('author_password', '暗証番号', $author_password, 'alnum'      , true);
            $validator->validate('author_password', '暗証番号', $author_password, '/^[\w]{4}$/', true);

            // トピックのパスワード認証
            if (!db_auth_topic4author_password($topic_id, md5($author_password)))
            {
                $validator->addErrors('author_password', '認証に失敗しました。');
            }

            // 有効チェック
            if (!$validator->isValid())
            {
                // エラーメッセージをクリア
                $validator->clearErrors();
                $validator->addErrors('author_password', '認証に失敗しました。');
            }
        }
    }
}

?>
